/* ----------------------------------------------------------------------
 * FICHERO : test_stress_sort.cpp
 *
 * DESCRIPCION : Programa de prueba del algoritmo Quick Sort en su versi•n
 *    no recursiva
 *
 * OBSERVACIONES :
 *
 * ----------------------------------------------------------------------*/
//#define __DEBUG_MODE 1

#include <parallel/basic_iterator.h>
#include <parallel/features.h>
#include <parallel/parallel.h>
#include <parallel/algorithm>
#include <omp.h>
#include <stdlib.h>
#include <iostream>
#include <sys/time.h>
#include <sys/resource.h>
#include <boost/countertree/util_sort.hpp>
#include <boost/countertree/util/time_measure.hpp>
#include <vector>
#include <random>
#include "tbb/tbb_stddef.h"
#include "tbb/task_scheduler_init.h"
#include "tbb/parallel_sort.h"



#define NELEM 100000000
using namespace std ;
//using namespace cntree ;

using boost::countertree::NThread ;
using boost::countertree::util::time_point ;
using boost::countertree::util::now;
using boost::countertree::util::subtract_time ;
using namespace __gnu_parallel;
//using namespace __gnu_parallel::parallel_tag;
typedef typename std::vector<uint64_t>::iterator iter_t ;

std::mt19937_64 my_rand(0);

void Prueba ( void (*func) (iter_t, iter_t, std::less<uint64_t>) );
vector<uint64_t> A  ;

int main ()
{   //--------------------------- Inicio --------------------------
    cout<<"max. OpenMP threads:"<<omp_get_max_threads();
    cout<<" (processors)"<< omp_get_num_procs()<<endl;
	A.reserve ( NELEM);

    //------------------------ Inicio -----------------------------
    cout<<"Sort of "<< NELEM<<" uint64_t elements in a vector -----------\n" ;

    cout<<"Random elements, few repeated -----------------------\n";
    A.clear();
    for ( uint32_t i = 0 ; i < NELEM  ; i ++)
        A.push_back(my_rand()) ;
    cout<<"std::stable_sort\n";
    Prueba(&std::stable_sort ) ;
    cout<<"countertree::stable_sort\n";
    Prueba(&boost::countertree::stable_sort) ;


    return 0 ;
};

void Prueba ( void (*func) (iter_t, iter_t, std::less<uint64_t>) )
{   //---------------------------- Inicio --------------------------------

	std::vector<uint64_t>  B(A) ;

	double duracion ;
	time_point start, finish;
    NThread NT ;
    std::vector<iter_t> Vit;
    size_t Cupo = NELEM / NT() ;
    for ( uint32_t i =1 ; i < NT() ; ++i)
    {   Vit.push_back ( B.begin() + (i*Cupo));
    };
    Vit.push_back( B.begin() + NELEM);

    //Sort the parts with 1 thread
    for ( uint32_t i =0  ; i < NT() ; ++i)
    {   start= now() ;
        func (B.begin() + (i*Cupo) , Vit[i] , std::less<uint64_t>());
        finish = now() ;
        duracion = subtract_time(finish ,start) ;
        cout<<"Part "<<i<<" secs :"<<duracion<<"\n";
    };
    B = A ;
    //sort with all the threads
    std::vector<std::thread> VT ( NT());
    start= now() ;
    for ( uint32_t i =0  ; i < NT() ; ++i)
    {   VT[i] = std::thread (func, B.begin() + (i*Cupo) , Vit[i], std::less<uint64_t>());
    }

    for ( uint32_t i =0  ; i < NT() ; ++i)	VT[i].join() ;
    finish = now() ;
    duracion = subtract_time(finish ,start) ;
    cout<<"Parallel Sort  secs :"<<duracion<<"\n";
};
