//----------------------------------------------------------------------------
/// @file indirect_merge.hpp
/// @brief Insertion Sort algorithm
///
/// @author Copyright (c) 2010 Francisco Jose Tapia (fjtapia@gmail.com )\n
///         Distributed under the Boost Software License, Version 1.0.\n
///         ( See accompanyingfile LICENSE_1_0.txt or copy at
///           http://www.boost.org/LICENSE_1_0.txt  )
/// @version 0.1
///
/// @remarks This is an idea abiut the merge of two consecutive parts, using
///          pointers and moving the elements in the final step
//-----------------------------------------------------------------------------
#ifndef __CNTREE_SORT_INDIRECT_MERGE_HPP
#define __CNTREE_SORT_INDIRECT_MERGE_HPP

#include <functional>
#include <boost/countertree/util/util_iterator.hpp>
#include <boost/countertree/util/sort/insertion_sort.hpp>
#include <boost/countertree/util/sort/token.hpp>
#include <vector>
#include <type_traits>

namespace boost
{
namespace countertree
{
namespace util
{
namespace sort
{

using namespace boost::countertree::util;

static const uint32_t MSortPos_min = 48 ;
/*
template < class iter_t,
            typename compare=std::less<typename iter_value<iter_t>::type >
void indirect_merge ( iter_t it, size_t N1 , size_t N2, size_t *VPos)
{   //------------------------------ begin ---------------------------------
    typedef typename iter_value<iter_t>::type value_t ;

    size_t Nx = N1 + N2 ;
    size_t Pos1 = 0, Pos2 = N1, Count =0 ;

    while (Pos1 < N1 and Pos2 < Nx )
    {   VPos[ Count++] = ( *(it+Pos1) < *(it+Pos2))? Pos1++: Pos2++ ;
    };
    while ( Pos1 < N1) VPos [ Count++] = Pos1++ ;
    while ( Pos2 < Nx) VPos [ Count++] = Pos2++ ;

    //------------------------------------------------------------------------
    //           Data movement
    //------------------------------------------------------------------------
    size_t  A = 0 , B =0 , Pos =0 ;
    iter_t itA,itB ;
    while ( Pos < Nx)
    {   for (; Pos < Nx and VPos[Pos] == Pos; ++Pos);
        if ( Pos == Nx) return ;
        A = B = Pos ;
        itA = it + A ;
        value_t  Aux ( std::move ( *itA ));
        while ( (B = VPos[A]) != Pos  )
        {   VPos[A] = A ;
            itB = it + B ;
            *itA = std::move ( *itB);
            itA = itB ;
            A = B ;
        };
        *itA = std::move ( Aux);
        VPos[A] =  A ;
        ++Pos ;
    }
};
*/
//---------------------------------------------------------------------------
//  function : merge_sort
/// @brief Merge Sort algorithm ( stable sort)
/// @param [in] first : first iterator to the elements to sort
/// @param [in] last : last iterator to the elements to sort
/// @param [in] P : pointer of a data area with at least (N+1)/2 elements
/// @exception
/// @return
/// @remarks
//---------------------------------------------------------------------------
template < class iter_t,
           typename compare=std::less<typename iter_value<iter_t>::type >
         >
void merge_sort_pos ( iter_t first , iter_t last  ,
                         compare comp, size_t *VPos )
{   //------------------------- begin ----------------------
    typedef typename iter_value<iter_t>::type value_t ;
    auto N = last - first ;
    assert ( N >= 0 );
    auto N1 = ( N+1)>>1 ;

    if ( N1 > MSortPos_min)
    {   merge_sort_pos ( first , first + N1, comp,VPos );
        merge_sort_pos ( first + N1 , last , comp, VPos );
    }
    else
    {   insertion_sort ( first , first + N1,comp );
        insertion_sort ( first + N1 , last ,comp );
    };
    //-----------------------------------------------------------------------

    size_t N2  = N -N1 ;
    size_t Nx = N1 + N2 ;
    size_t Pos1 = 0, Pos2 = N1, Count =0 ;

    while (Pos1 < N1 and Pos2 < Nx )
    {   VPos[ Count++] = comp(*(first+Pos1),*(first+Pos2))? Pos1++: Pos2++ ;
    };
    while ( Pos1 < N1) VPos [ Count++] = Pos1++ ;
    while ( Pos2 < Nx) VPos [ Count++] = Pos2++ ;

    //------------------------------------------------------------------------
    //           Data movement
    //------------------------------------------------------------------------
    size_t  A = 0 , B =0 , Pos =0 ;
    iter_t itA,itB ;
    while ( Pos < Nx)
    {   for (; Pos < Nx and VPos[Pos] == Pos; ++Pos);
        if ( Pos == Nx) return ;
        A = B = Pos ;
        itA = first + A ;
        value_t  Aux ( std::move ( *itA ));
        while ( (B = VPos[A]) != Pos  )
        {   VPos[A] = A ;
            itB = first + B ;
            *itA = std::move ( *itB);
            itA = itB ;
            A = B ;
        };
        *itA = std::move ( Aux);
        VPos[A] =  A ;
        ++Pos ;
    }
} ;

//---------------------------------------------------------------------------
//  function : merge_sort
/// @brief Merge Sort algorithm ( stable sort)
/// @param [in] first : first iterator to the elements to sort
/// @param [in] last : last iterator to the elements to sort
/// @param [in] P : pointer of a data area with at least (N+1)/2 elements
/// @exception
/// @return
/// @remarks
//---------------------------------------------------------------------------
template < class iter_t,
           typename compare = std::less<typename iter_value<iter_t>::type>
         >
void merge_sort_pos (iter_t first, iter_t last, compare comp = compare() )
{   //-------------------------- begin ------------------------------
    auto N = last - first ;
    assert ( N >= 0 );
    if ( N < MSortPos_min)
    {   insertion_sort ( first, last , comp);
        return ;
    }

    void * Ptr = ::operator new ( N * sizeof (size_t));
    if ( Ptr == nullptr) throw std::bad_alloc ( );

    merge_sort_pos ( first , last , comp,static_cast <size_t *> (Ptr) );
    ::operator delete ( Ptr) ;

};


//
//****************************************************************************
};//    End namespace sort
};//    End namespace util
};//    End namespace countertree
};//    End namespace boost
//****************************************************************************
//
#endif
